/*
  Emulates Treasure Seeker Classic which represents
all rooms based on memory thereof.

  Replaces the UpdateIcon proc with a system that
avoids use of overlays, to reduce object usage.
An accompanying 'rooms.dmi' is present with all
possible directions therein.
*/

var/const/TURF_MEMORY_LAYER = FLOAT_LAYER

/turf
	icon = 'rooms.dmi'
	var
		tmp/image/black_overlay
		memorize = 1

	New()
		src.tag = "\ref[src]"
		black_overlay = image('blackoverlay.dmi', loc = src, layer = TURF_MEMORY_LAYER)
		..()

	Enter(atom/movable/O)
		. = ..()
		if(.)
			if(istype(O,/mob/player)) if(O:key) src.Memorize(O)


	UpdateIcon()
		var/exits = 0
		if(src.exits["north"] & EXIT_OPEN) exits += NORTH
		if(src.exits["south"] & EXIT_OPEN) exits += SOUTH
		if(src.exits["east"]  & EXIT_OPEN) exits += EAST
		if(src.exits["west"]  & EXIT_OPEN) exits += WEST
		if(src.exits["up"]    & EXIT_OPEN) exits += UP
		if(src.exits["down"]  & EXIT_OPEN) exits += DOWN

		src.icon = 'rooms.dmi'
		src.icon_state = "[exits]"

		for(var/mob/player/M in src) Memorize(src)


	proc/Memorize(mob/player/M)
		spawn(1)
			var/exits = 0
			if(src.exits["north"] & EXIT_OPEN) exits += NORTH
			if(src.exits["south"] & EXIT_OPEN) exits += SOUTH
			if(src.exits["east"]  & EXIT_OPEN) exits += EAST
			if(src.exits["west"]  & EXIT_OPEN) exits += WEST
			if(src.exits["up"]    & EXIT_OPEN) exits += UP
			if(src.exits["down"]  & EXIT_OPEN) exits += DOWN

			var/image/memoryoverlay = image(icon = 'rooms.dmi', icon_state = "[exits]",
				loc = src, layer = TURF_MEMORY_LAYER)

			if(M.memory.Find(src.tag))
				var/image/doomed = M.turf_memory["[src.tag]"]
				del(doomed)
			else
				if(M.client) M.client.images -= src.black_overlay

			M.memory += src.tag
			M.turf_memory["[src.tag]"] = memoryoverlay
			M << memoryoverlay


mob/player
	sight = SEE_TURFS
	see_in_dark = 1

	var/tmp/list/turf_memory = list()
	var/tmp/list/memory = list()

	Login()
		..()
		for(var/turf/T in world)
			if(T.memorize)
				src.client.images += T.black_overlay